//----------------------------------------------------------------------
//   Copyright 2007-2008 SiMantis Inc.
//   All Rights Reserved Worldwide
//
//   Licensed under the Apache License, Version 2.0 (the
//   "License"); you may not use this file except in
//   compliance with the License.  You may obtain a copy of
//   the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in
//   writing, software distributed under the License is
//   distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
//   CONDITIONS OF ANY KIND, either express or implied.  See
//   the License for the specific language governing
//   permissions and limitations under the License.
//----------------------------------------------------------------------

//----------------------------------------------------------------------------
//
// Program 10.7:  Constraint distribution operator
//
//----------------------------------------------------------------------------
// 
import uvm_pkg::*;
`include "uvm_macros.svh"

class rand_c;   // /sv/SiMantis/examples/book_examples/program_10.7.sv
	rand integer a,b;
	rand bit reset;
	// reset ==1'b1 1% of the time 
	// reset ==1'b0  99% of the time
	constraint c_reset {reset dist {1'b1:=1, 1'b0:=99};}
	
	// a==1 @weight=1; a==2 @weight=3, a==4 to 5  @weight=5,  
	// a==value(b to b+2) EACH @weight=number of entries in this range /3
	// total weight= 1+3+5+5+1+1+1 = 17;   
	//  Prob (a==1) is 1/17; prob(a==4) is  5/17
	constraint cst_c1 {a dist {1:=1, 2:=3, [4:5]:=5, [b:b+2]:/3};}
	// Also legal 
	// constraint cst_c1 { a dist {1:=1, 2:=3, [3:500]:=5, [b:b+2]:/3};
	//                     a inside {[2:410], 450, [475:480]};	}  
endclass : rand_c
module m; 
	rand_c obj=new(); 
	initial begin : test
		repeat(25) begin : r1
			if (!randomize(obj)) `uvm_error("obj", "Randomize failed"); 
			`uvm_info("obj", $sformatf("obj.a=%h, obj.b=%h, obj.reset=%b", 
						obj.a, obj.b, obj.reset), UVM_MEDIUM);	
		end : r1
	end : test
endmodule : m

/*	  
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=ef71cd8f, obj.b=ef71cd8f, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=5fffe562, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000001, obj.b=4e5c7d71, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=b63ffe72, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000002, obj.b=8faf6bda, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=073417fe, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000004, obj.b=3fa598de, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=6915e6d2, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=6e54441d, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000004, obj.b=a46c79da, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=ac8717c2, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=ff106774, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000002, obj.b=997a9b25, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000002, obj.b=790ea07f, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000002, obj.b=8243b1b9, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=cb89af26, obj.b=cb89af24, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=b45f8585, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000004, obj.b=5872c3f6, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=fa2c4f22, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000005, obj.b=295afc23, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000002, obj.b=7f2224a9, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000002, obj.b=7dd45dce, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000004, obj.b=e2b033b4, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000004, obj.b=1a2d8099, obj.reset=0
# UVM_INFO program_10.7.sv(50) @ 0: reporter [obj] obj.a=00000002, obj.b=4a241429, obj.reset=0 */ 



