import counter_pkg::*; import uvm_pkg::*; `include "uvm_macros.svh" 
class data_item;
    rand logic[3:0] data;
endclass : data_item

class transaction extends data_item;
    rand logic ld=0;
    int reset_cycles, idle_cycles;
    rand ct_scen_e kind;
    // constraint values { ld dist {1'b0 := 80, 1'b1 := 20}; } // 0 80% of time
    constraint data_range { data > 2 && data <=9; } 
    constraint cst_xact_kind {
        kind dist {
            CT_LOAD := 5, 
            CT_RESET := 2, 
            CT_WAIT := 97,
            CT_DONE := 1
        };
    } // cst_xact_kind
endclass : transaction

class simple_random_sequence;
    transaction tx=new();
    // randomize 
    virtual task run();
        ap_rand_seq: assert(randomize(tx)); 
    endtask : run
endclass : simple_random_sequence

 

module counter_max_tb_simple;
    parameter MAX_COUNT_TB=6, MIN_COUNT_TB=3; 
    // defparam counter_max1.MIN_COUNT=3; 
    // illegal because have parameter port list
    logic[3:0] data_in=0, data=0;
    logic ld=0;
    logic[3:0] counter, counter2;
    logic clk=1'b1, rst_n=1'b1;
    simple_random_sequence seq=new();
    counter_max #(.MAX_COUNT(MAX_COUNT_TB), 
        .MIN_COUNT(MIN_COUNT_TB)) counter_max1(.*); 
    // counter_max counter_max2(.counter(counter2), .*); 
    initial forever #10 clk=!clk; 
    
    // test using simple_random_sequence
    always @ (negedge clk) begin : random1
        rst_n <= 1'b1; ld <=1'b0; 
        repeat(3) @ (negedge clk) rst_n <= 1'b0; 
        repeat(3) @ (negedge clk)rst_n <= 1'b1;
        // dada and load 
        for (int i=0; i<100; i++) begin : for1
        	@ (negedge clk);
            seq.run(); 
            case (seq.tx.kind)
                CT_LOAD : begin 
                    // $display("PUSH"); 
                    //`ovm_info({get_type_name(),":drive_dut"},{"rsp_txn - pre",rsp_txn.convert2string()},OVM_LOW)
                    load_task(seq.tx.data); 
                end
                CT_RESET : begin 
                    // $display("POP"); 
                    //`ovm_info({get_type_name(),":drive_dut"},{"rsp_txn - pre",rsp_txn.convert2string()},OVM_LOW)
                    reset_task(); 
                end 
                CT_WAIT : begin 
                    //$display("PUSH_POP"); 
                    //`ovm_info({get_type_name(),":drive_dut"},{"rsp_txn -pre",rsp_txn.convert2string()},OVM_LOW)
                    idle_task(seq.tx.data); 
                end
                CT_DONE : begin 
                    // $display("IDLE");
                    //`ovm_info({get_type_name(),":drive_dut"},{"rsp_txn -pre",rsp_txn.convert2string()},OVM_LOW)
                    done_task(); 
                end // seq.tx.idle_cycles);
            endcase
            // @ (negedge clk); 
        end : for1
        `uvm_info("counter_max", "End of test", UVM_MEDIUM); 
        `uvm_info("counter_max", $sformatf("counter_max1.cover_count=%d, counter_max1.fail_count=%d", 
            counter_max1.cover_count, counter_max1.fail_count), UVM_MEDIUM);
        $finish;
    end : random1
    
    // load counter
    task load_task(int data);
        data_in <= data;
        ld <= 1'b1; 
        @ (negedge clk)ld <= 1'b0; 
        
    endtask : load_task
    
    // reset
    task reset_task();
        ld <= 1'b0; 
        repeat(1) rst_n <= 1'b0; 
        @ (negedge clk)rst_n <= 1'b1;
    endtask : reset_task
    
    // IDLE
    task idle_task(int data);// for now, make idle for 4 cycles 
        automatic logic[3:0] v=data; 
        for (int i=0; i<4; i++) begin
            ld <= 1'b0; 
            if (!randomize(v)) `uvm_error("MYERR", "This is a randomize error")
            data_in <= v;
            @ (negedge clk); 
        end
    endtask : idle_task
    
    // DONE, for now, a do nothing 
    task done_task();
        @ (negedge clk); 
        
    endtask : done_task
endmodule : counter_max_tb_simple 
