import uvm_pkg::*;
`include "uvm_macros.svh"

module simple; // testbench_ex.sv
   bit clk, req, ack; 
   string tID="simple";
   ap_req_ack: assert property(@(posedge clk)  
                      req |=> ##[1:5] ack) else 
   	    `uvm_info(tID,$sformatf("%m : error in req ack %b %b", req, ack), UVM_MEDIUM)
                      
   initial forever #5 clk=!clk; 
   initial begin 
     repeat(200) begin 
       @(posedge clk); 
       if (!randomize(req, ack)) `uvm_error("MYERR", "This is a randomize error"); 
       $display("%t %b %b", $time, req, ack); 
       end 
       $finish; 
    end
endmodule 


import uvm_pkg::*;
`include "uvm_macros.svh"

module simple_ct; // testbench_ex.sv
   bit clk, req, ack; 
   string tID="simple";
   ap_req_ack: assert property(@(posedge clk)  
                      req |=> ##[1:5] ack) else 
   	    `uvm_info(tID,$sformatf("%m : error in req ack %b %b", req, ack), UVM_MEDIUM)
                      
   initial forever #5 clk=!clk; 
   initial begin 
     repeat(200) begin 
       @(posedge clk);  //with { a < b ; a + b < length ; };
       if (!randomize(req, ack)  with 
           { req dist {1'b1:=1, 1'b0:=3};
       		 ack dist {1'b0:=1, 1'b0:=5};})
       	          `uvm_error("MYERR", "This is a randomize error")
       $display("%t %b %b", $time, req, ack); 
       end 
       $finish; 
    end
endmodule 


import uvm_pkg::*;
`include "uvm_macros.svh"

class c; 
	rand bit req, ack; 
	constraint req_ct { req dist {1'b1:=1, 1'b0:=3};}
	constraint ack_ct { ack dist {1'b1:=1, 1'b0:=5};}
endclass 


module class_based; // testbench_ex.sv
   bit clk, req, ack; 
   string tID="simple";
   c c1; 
   ap_req_ack: assert property(@(posedge clk)  
                      req |=> ##[1:5] ack) else 
   	    `uvm_info(tID,$sformatf("%m : error in req ack %b %b", req, ack), UVM_MEDIUM)
                      
   initial forever #5 clk=!clk; 
   initial begin 
   	 c1=new();
     repeat(200) begin 
       @(posedge clk);  //with { a < b ; a + b < length ; };
       if (!c1.randomize()) 
       	          `uvm_error("MYERR", "This is a randomize error")
       req <= c1.req; 
       ack <= c1.ack;
       $display("%t %b %b", $time, req, ack); 
       end 
       $finish; 
    end
endmodule 
